
import sys
sys.path.append('../')
import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_rockets():
    rockets_data = ''
    rockets_result = keyOrder(alphaOrder(
        readJSONFile('rockets/all.json')), 'rocket_id')
    try:
        rockets_data = keyOrder(
            alphaOrder(spacexpython.rockets.allrockets('', 1)),
            'rocket_id')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on all rockets")
    assert rockets_data == rockets_result


def test_rocketsP():
    rockets_data = ''
    rockets_result = keyOrder(alphaOrder(
        readJSONFile('rockets/all_limit1.json')), 'rocket_id')
    try:
        rockets_data = keyOrder(alphaOrder(
            spacexpython.rockets.allrockets('{"limit":"1"}', 1)),
            'rocket_id')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on all rockets")
    assert rockets_data == rockets_result


def test_rocketF1():
    f1_data = ''
    f1_result = alphaOrder(readJSONFile('rockets/falcon1.json'))
    try:
        f1_data = alphaOrder(
            spacexpython.rockets.rocket('falcon1', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on Falcon1")
    assert f1_data == f1_result


def test_rocketF9():
    f9_data = ''
    f9_result = alphaOrder(readJSONFile('rockets/falcon9.json'))
    try:
        f9_data = alphaOrder(
            spacexpython.rockets.rocket('falcon9', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on Falcon9")
    assert f9_data == f9_result


def test_rocketBFR():
    # BFR has been renamed Starship
    # This function has been marked as deprecated
    bfr_data = ''
    bfr_result = alphaOrder(readJSONFile('rockets/starship.json'))
    try:
        bfr_data = alphaOrder(
            spacexpython.rockets.rocket('starship', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on BFR(now starship)")
    assert bfr_data == bfr_result


def test_rocketstarship():
    starship_data = ''
    starship_result = alphaOrder(readJSONFile('rockets/starship.json'))
    try:
        starship_data = alphaOrder(
            spacexpython.rockets.rocket('starship', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on Starship (was BFR)")
    assert starship_data == starship_result


def test_rocketFH():
    fh_data = ''
    fh_result = alphaOrder(readJSONFile('rockets/falconheavy.json'))
    try:
        fh_data = alphaOrder(
            spacexpython.rockets.rocket('falconheavy', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on Falcon Heavy")
    assert fh_data == fh_result



import sys
sys.path.append('../')
import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_crew():
    crew_data = ''
    crew_result = '[]'
    try:
        crew_data = alphaOrder(spacexpython.crew.crew())
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on Crew")
    assert crew_result == crew_data




"""urldata module

This is a module defines the endpoints for the REST SpaceX API
as well as some global data
"""


class Domain(object):
    """ Base URL from which to assemble request URLs """
    base = "https://api.spacexdata.com"

    """ API Version """
    version = "v3"
    main = base + "/" + version + "/"

    """ Declaration of the endpoints

        Launches Information """
    main_launches = "launches"
    latest_launches = main_launches + "/latest"
    next_launches = main_launches + "/next"
    upcoming_launches = main_launches + "/upcoming"
    past_launches = main_launches + "/past"

    """ Company/API information """
    main_info = "info"
    main_api = ""

    """ Rocket information """
    main_rockets = "rockets"
    falcon1 = main_rockets + "/falcon1"
    falcon9 = main_rockets + "/falcon9"
    falconheavy = main_rockets + "/falconheavy"
    bigfalconrocket = main_rockets + "/bfr"
    bigfalconrocket = main_rockets + "/starship"

    """ Capsule Information """
    main_capsules = "capsules"
    upcoming_capsules = main_capsules + "/upcoming"
    past_capsules = main_capsules + "/past"

    """ Cores Information """
    main_cores = "cores"
    upcoming_cores = main_cores + "/upcoming"
    past_cores = main_cores + "/past"

    """ Dragons Information """
    main_dragons = "dragons"

    """ Landing Pads Information """
    main_landingpads = "landpads"

    """ History Information"""
    main_history = "history"

    """ Launch Pads Information """
    main_launchpads = "launchpads"

    """ Mission Information """
    main_missions = "missions"

    """ Payload Information """
    main_payloads = "payloads"

    """ Roadster Information """
    main_roadster = "roadster"

    """ Ships Information """
    main_ships = "ships"

    """ Crew Information """
    main_crew = "crew"



"""History module

This is a module which allows wrapper access to the history of SpaceX;
specifically events which have occurred in the company's timeline

This file is imported as a module and contains the following
functions:

    * history - returns all history information
    * one - returns information about a specific event

"""
from . import urldata
from . import utils


def history(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about the SpaceX timeline

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all historical
        events respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_history
    return utils.makeRequest(requestUrl, timeOut, parameters)


def one(event='', parameters='', timeOut=1):
    """
    :type event: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about the SpaceX timeline

    Parameters
    ----------
    event : ID of the event of interest (str)
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of a specific
     historical event respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_history + \
        "/" + event
    return utils.makeRequest(requestUrl, timeOut, parameters)



"""Landingpads module

This is a module which allows wrapper access to information concerning
the landing pads that SpaceX uses to allow its' rocket cores to land
ready for refurbishment and reuse. Note that this does NOT include
drone ships

This file is imported as a module and contains the following
functions:

    * landingpads - returns all landing pad information
    * one - returns information about a specific pad

"""
from . import urldata
from . import utils


def landingpads(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about the SpaceX landing pads

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all landing pads
        used by SpaceX respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + \
            urldata.Domain.main_landingpads
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)


def one(pad='', parameters='', timeOut=1):
    """
    :type pad: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about a specific SpaceX landing pad

    Parameters
    ----------
    pad: landing pad of interest (str)
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of a single landing pad
        used by SpaceX respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + \
            urldata.Domain.main_landingpads + "/" + pad
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)




import sys
sys.path.append('../')

import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_launchpads():
    launchpads_data = ''
    launchpads_result = keyOrder(
        alphaOrder(readJSONFile('launchpads/all.json')), 'id')
    try:
        launchpads_data = keyOrder(alphaOrder(
            spacexpython.launchpads.launchpads('', 1)), 'id')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on launchpads.alllaunchpads")
    assert launchpads_data == launchpads_result


def test_one_launchpad():
    one_data = ''
    one_result = alphaOrder(
        readJSONFile('launchpads/onepad4.json'))
    try:
        one_data = alphaOrder(
            spacexpython.launchpads.one('ksc_lc_39a', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on launchpads.one('ksc_lc_39a')")
    assert one_data == one_result



"""Roadster module

This is a module which allows wrapper access to information about Elon
Musk's Tesla Model 3 Roadster which is now an artificial satellite of
the Sun.

This file is imported as a module and contains the following
function:

    * roadster - returns all roadster information

"""
from . import urldata
from . import utils


def roadster(timeOut=1):
    """

    :type timeOut: Optional[int]

    Returns details about the Tesla Model 3 Roadster

    Parameters
    ----------

    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of the Tesla Model 3
        Roadster, including position, speed etc
    """
    utils.validateParameters('', __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_roadster
    return utils.makeRequest(requestUrl, timeOut)



"""Missions module

This is a module which allows wrapper access to the functions within
the SpaceX API to return information about SpaceX missions

This file is imported as a module and contains the following
functions:

    * missions - returns all mission information
    * one - returns information about a specific mission

"""
from . import urldata
from . import utils


def missions(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about all missions

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all missions
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + urldata.Domain.main_missions
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)


def one(mission='', parameters='', timeOut=1):

    """
    :type mission: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about a specific mission

    Parameters
    ----------
    mission : Mission identifier (str)
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of a single mission
        respecting any filters
    """

    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + \
            urldata.Domain.main_missions + "/" + mission
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)



"""Utilities module

This is a utilities module with common functionality
which is used by many other modules.

This file is imported as a module and contains the following
functions:

    * jsonParameters - converts a JSON document
                        into a URL parameter string
    * makeRequest - function to call a REST api
    * getAPISupporting - Get information from API support page
    * validateParameters - validate parameters to any API functions

    * Internal Functions:
        -   func_name
        -   clients
        -   apps

"""
import ast
import json
import sys
import requests
import cachepy
from bs4 import BeautifulSoup
from tinydb import TinyDB, Query
from tinydb.storages import MemoryStorage


sys.path.append('../')

from .exceptions import *


clientcache = cachepy.Cache()
APIcache = cachepy.Cache()
appcache = cachepy.Cache()


def jsonParameters(parameters):
    """

    :type parameters: str

    converts a JSON document into a URL parameter string

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing the list
        of parameters to add to the API call

    Returns
    -------
    parms
        a string which can be appended to the URL
    """
    if (parameters == ''):
        return ''
    else:
        jsonObject = json.loads(parameters)
        parms = '?'
        for key in jsonObject:
            value = jsonObject[key]
            parms = parms + key + '=' + value + '&'
        return parms[:-1]


def makeRequest(requestUrl, timeOut=1, parameters=''):
    """
    :param requestUrl: str
    :param timeOut: Optional[str]
    :param parameters: Optional[str]
    :return: object:

    Sends a request to the API

    Parameters
    ----------
    requestURL : str
        string to pass into the REST API
    timeOut : int
        optional - API call timeout
    parameters : str
        optional parameters to append to URL as query modifiers

    Returns
    -------
    response
        a string which is a JSON document returned from the API

    Exceptions
    ----------
    SpaceXReadTimeOut
        raised when the API call breaches the timeout limit
    """

    try:
        url_response = requests.get(url=str(requestUrl)
                                    + jsonParameters(parameters),
                                    timeout=timeOut)
    except requests.exceptions.ReadTimeout:
        raise SpaceXReadTimeOut('Space/X Timeout Error')
    else:
        response = url_response.json()
    return response


@clientcache
def buildclients(url_response):
    """
    :param url_response: str

    Parameters
    ----------
    :param url_response: str

    Returns
    -------
    clientDB
        An in-memory TinyDB instance containing clients

    """
    # Storage for clients will be in an in-memory TinyDB
    clientDB = TinyDB(storage=MemoryStorage)

    page = BeautifulSoup(url_response.text, 'html.parser')
    table = page.find('tbody')

    for trow in table.find_all('tr')[0:]:
        tds = trow.find_all('td')

        # Split languages and form JSON Array
        initLangs = tds[1].text.split("/")
        languages = "["
        for lang in initLangs:
            languages = languages + '"' + lang.strip() + '",'
        languages = languages[:-1] + "]"

        # Split repo types and form JSON Array
        initRepos = tds[3].text.split(",")
        repotypes = "["
        for repo in initRepos:
            repotypes = repotypes + '"' + repo.strip() + '",'
        repotypes = repotypes[:-1] + "]"

        # Split Creators and form JSON Array
        initcreators = tds[2].text.split(",")
        creators = "["
        for creator in initcreators:
            creators = creators + '"' + creator.strip() + '",'
        creators = creators[:-1] + "]"

        # Split links and form JSON Array
        lnks = ''
        for lnk in tds[3].find_all('a', href=True):
            lnks = lnks + ',' + '"' + lnk['href'] + '"'
        lnks = '[' + lnks[1:] + "]"

        # insert the record into the database
        record = '{"Name":"' + tds[0].text + '","Languages":' \
                 + languages + ',"Creators":' + creators \
                 + ',"Repos":' + repotypes \
                 + ',"ReposLinks":' + lnks + '}'
        clientDB.insert(json.loads(record))

    return clientDB


@appcache
def buildapps(url_response):
    """
    :param url_response: str

    Parameters
    ----------
    :param url_response: str

    Returns
    -------
    appsDB
        An in-memory TinyDB instance containing apps

    """
    # Storage for apps will be in an in-memory TinyDB
    appsDB = TinyDB(storage=MemoryStorage)

    page = BeautifulSoup(url_response.text, 'html.parser')
    name_box = page.find('tbody')
    print(name_box)
    for trow in name_box.find_all('tr')[0:]:
        tds = trow.find_all('td')

        # Get links for app/website etc and form JSON Array
        lnks = ''
        for lnk in tds[0].find_all('a', href=True):
            lnks = lnks + ',' + '"' + lnk['href'] + '"'
        lnks = '[' + lnks[1:] + "]"

        type = ''
        platform = ''
        # Separate Types out and deal with platforms:
        if ("WEBSITE" in (tds[1].text).upper()):
            type = type + ',"WEBSITE"'
            platform = platform + ',"WEB"'
            
        if ("ALEXA SKILL" in (tds[1].text).upper()):
            type = type + ',"ALEXA"'
            platform = platform + ',"ALEXA"'
        
        if ("ANDROID APP" in (tds[1].text).upper()):
            type = type + ',"APP"'
            platform = platform + ',"ANDROID"'

        if ("IOS APP" in (tds[1].text).upper()):
            type = type + ',"APP"'
            platform = platform + ',"IOS"'

        if ("DISCORD BOT" in (tds[1].text).upper()):
            type = type + ',"BOT"'
            platform = platform + ',"DISCORD"'

        if ("API" in (tds[1].text).upper()):
            type = type + ',"API"'
            platform = platform + ',"API"'

        if ("APPS" in (tds[1].text).upper()):
            type = type + ',"APP"'
            platform = platform + ',"IOS","ANDROID"'

        type = '[' + type[1:] + "]"
        platform = '[' + platform[1:] + "]"

        # Split Creators and form JSON Array
        initcreators = tds[2].text.split(",")
        creators = "["
        for creator in initcreators:
            creators = creators + '"' + creator.strip() + '",'
        creators = creators[:-1] + "]"

        # Split "More" and form JSON Array
        initmore = tds[4].text.split(",")
        mores = '['
        for more in initmore:
            mores = mores + '"' + more.strip().replace("\""," ") + '",'
        mores = mores[:-1] + "]"
        if (mores == '[""]'):
            mores = '[]'

        # Get links for app/website etc and form JSON Array
        clnks = ''
        for lnk in tds[2].find_all('a', href=True):
            clnks = clnks + ',' + '"' + lnk['href'] + '"'
        clnks = '[' + clnks[1:] + "]"

        app = tds[0].text

        # Split "Repos" and form JSON Array
        initrepo = tds[3].text.split(",")
        repos = '['
        for repo in initrepo:
            repos = repos + '"' + repos.strip() + '",'
        repos = repos[:-1] + "]"
        if (repos == '[""]'):
            repos = '[]'

        # Get links for repo and form JSON Array
        rlnks = ''
        for lnk in tds[3].find_all('a', href=True):
            rlnks = rlnks + ',' + '"' + lnk['href'] + '"'
        rlnks = '[' + rlnks[1:] + "]"

        # Get more links for repo and form JSON Array
        mlnks = ''
        for lnk in tds[4].find_all('a', href=True):
            mlnks = mlnks + ',' + '"' + lnk['href'] + '"'
        mlnks = '[' + mlnks[1:] + "]"

        record = '{"Name":"' + app + '","Links":' \
                 + lnks + ',"Types":' + type \
                 + ',"Platforms":' + platform \
                 + ',"Creators":' + creators \
                 + ',"CreatorsLinks":' + clnks \
                 + ',"Repos":' + rlnks \
                 + ',"ReposLinks":' + rlnks \
                 + ',"More":' + mores \
                 + ',"MoreLinks":' + mlnks + '}'
        appsDB.insert(json.loads(record))
    return appsDB


@APIcache
def getAPISupporting(req, parameters, timeOut=1):
    """
    :param req: str
    :param parameters: Optional[str]
    :param timeOut: Optional[str]

    Parameters
    ----------
    req : str
        whether clients or apps are required
    parameters : str
        optional parameters use as query modifiers
    timeOut : int
        optional - API call timeout

    Returns
    -------
    response
        a string which is a JSON document returned from the
        APISupport page

    Exceptions
    ----------
    SpaceXReadTimeOut
        raised when the API call breaches the timeout limit
    """

    global clientDB
    global appsDB

    base = "https://github.com/r-spacex/SpaceX-API/blob/master/docs/"
    requestUrl = base + req + ".md"
    try:
        url_response = requests.get(url=str(requestUrl),
                                    timeout=timeOut)
    except requests.exceptions.ReadTimeout:
        raise SpaceXReadTimeOut('Space/X Timeout Error')
    else:
        if (req == 'clients'):
            # If the list of clients hasnt been built, build it
            try:
                # noinspection PyUnresolvedReferences
                clientDB
            except (NameError, UnboundLocalError):
                clientDB = buildclients(url_response)

            if (parameters == ''):
                # Make sure that here we read all the
                # records from the database into a JSON string
                responseT = str(clientDB.all())
                response = responseT.replace("'", "\"")
            else:
                # When there are parameters:
                Row = Query()  # noqa
                query = ''
                parametersJSON = json.loads(parameters)

                for key, value in parametersJSON.items():
                    query = query + "(Row." + str(key).capitalize() \
                        + ".any(" + str(value) + ")) & "
                query = query[:-2]
                print(query)
                responseT = clientDB.search(eval(query))
                response = str(responseT).replace("'", "\"")

        if (req == 'apps'):
            # If the list of apps hasnt been built, build it
            try:
                # noinspection PyUnresolvedReferences
                appsDB
            except (NameError, UnboundLocalError):
                appsDB = buildapps(url_response)

            if (parameters == ''):
                # Make sure that here we read all the
                # records from the database into a JSON string
                responseT = str(appsDB.all())
                response = str(responseT).replace("'", "\"")
            else:
                # When there are parameters:
                Row = Query()  # noqa
                query = ''
                parametersJSON = json.loads(parameters)

                for key, value in parametersJSON.items():
                    query = query + "(Row." + str(key).capitalize() \
                        + ".any(" + str(value) + ")) & "
                query = query[:-2]
                print(query)
                responseT = appsDB.search(eval(query))
                response = str(responseT).replace("'", "\"")

        if ((req != 'apps') and (req != "clients") and (req != '')):
            raise SpaceXParameterError

    return json.loads(response)


def validateParameters(inParameters, inFunction, subfunction):
    # If there are no parameters, then nothing needs validating.
    if (inParameters == ''):
        return True
    parameters = ast.literal_eval(inParameters)
    # Get the function name from the call
    discard, function = inFunction.split('.')

    # Open the database
    db = TinyDB('matrixDB.json')
    # Get the list of rows for this function/subfunction
    Row = Query()
    subFunctionLine = db.get((Row.function == function)
                             & (Row.subfunction == subfunction))

    functionParameters = subFunctionLine.get("parameters")
    fp = []
    ft = []

    cfp = 0
    # get list of parameters in function/subfunction
    # and populate 2 lists with their names and types
    for i in functionParameters:
        cfp = cfp + 1
        fp.append(i.get("parameter"))
        ft.append(i.get("parameter") + ".<class '"
                  + i.get("type") + "'>")

    # if there are no parameters applicable
    # for this function, return true
    if (cfp == 0):
        return True

    # Go though list of supplied parameters,
    # testing each for name validity and type validity
    for key, value in parameters.items():
        if key not in fp:
            raise SpaceXParameterError(key
                                       + " is not a valid parameter for "  # noqa
                                       + function + "."
                                       + subfunction)
        else:

            for g in ft:
                p, t = g.split('.')
                if (p == key):
                    break

            if (t == "<class 'int'>"):
                try:
                    attempt = int(value)
                    break
                except ValueError:
                    raise SpaceXParameterError("Type '"
                                               + str(type(value))
                                               .replace("<class '", "")
                                               .replace("'>", "")
                                               + "' is not valid for "
                                               + function + "."
                                               + subfunction
                                               + "(parameter: "
                                               + key + ")")

            if (t == "<class 'bool'>"):
                if (value.upper() not in ['TRUE', 'FALSE']):
                    raise SpaceXParameterError("Type '"
                                               + str(type(value)).
                                               replace("<class '", "").
                                               replace("'>", "")
                                               + "' is not valid for "
                                               + function + "."
                                               + subfunction
                                               + "(parameter: "
                                               + key + ")")
            else:
                # if the parameter is of the incorrect
                # type then raise an exception
                if (key + "." + str(type(value))) not in ft:
                    raise SpaceXParameterError("Type '"
                                               + str(type(value)).
                                               replace("<class '", "").
                                               replace("'>", "")
                                               + "' is not valid for "
                                               + function + "."
                                               + subfunction
                                               + "(parameter: "
                                               + key + ")")
    # If every parameter and type combination work out then good to go !
    return True


def func_name():
    # Source: https://stackoverflow.com/questions/5067604/determine-function-name-from-within-that-function-without-using-traceback  # noqa
    currentFuncName = lambda n=0: sys._getframe(n + 1).f_code.co_name
    return currentFuncName(1)



"""Capsules module

This is a module which allows wrapper access to the functions within
the SpaceX API to return information about the capsules within
SpaceX's fleet

This file is imported as a module and contains the following
functions:

    * capsules - returns all capsules
    * upcoming - returns information about upcoming capsules
    * past - returns information about past capsules
    * one - returns information about a specific capsule

"""
from . import urldata
from . import utils


def capsules(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about all capsules

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the capsules
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_capsules
    return utils.makeRequest(requestUrl, timeOut, parameters)


def upcoming(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about upcoming capsules

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the
        upcoming capsules respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.upcoming_capsules
    return utils.makeRequest(requestUrl, timeOut, parameters)


def past(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about past capsules

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the
        past capsules respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.past_capsules
    return utils.makeRequest(requestUrl, timeOut, parameters)


def one(capsule_id, parameters='', timeOut=1):
    """

    :type capsule_id: [str]
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about a specific capsule

    Parameters
    ----------
    capsule_id : ID of the capsule of interest
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the
        capsule respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_capsules + \
        "/" + str(capsule_id)
    return utils.makeRequest(requestUrl, timeOut, parameters)




import sys
sys.path.append('../')
import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_cores():
    cores_data = ''
    cores_result = keyOrder(alphaOrder(readJSONFile(
        'cores/cores.json')), 'core_serial')
    try:
        cores_data = keyOrder(alphaOrder(
            spacexpython.cores.cores('', 1)), 'core_serial')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on cores")
    assert cores_data == cores_result


def test_upcoming():
    upcoming_data = ''
    upcoming_result = keyOrder(alphaOrder(readJSONFile(
        'cores/upcoming.json')), 'core_serial')
    try:
        upcoming_data = keyOrder(alphaOrder(
            spacexpython.cores.upcoming()), 'core_serial')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on upcoming cores")
    assert upcoming_data == upcoming_result


def test_past():
    past_data = ''
    past_result = keyOrder(alphaOrder(readJSONFile(
        'cores/past.json')), 'core_serial')
    try:
        past_data = keyOrder(alphaOrder(
            spacexpython.cores.past()), 'core_serial')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on past cores")
    assert past_data == past_result


def test_one():
    one_result = alphaOrder(readJSONFile(
        'cores/one_B1032.json'))
    try:
        one_data = alphaOrder(spacexpython.cores.one('B1032'))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on one core (B1032)")
    assert one_data == one_result


def test_coresP():
    coresP_data = ''
    coresP_result = alphaOrder(readJSONFile
                               ('cores/coresP.json'))
    try:
        coresP_data = alphaOrder(
            spacexpython.cores.cores(
                '{"core_serial":"B1037","block":"3"}'))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on coresP")
    assert coresP_data == coresP_result



"""Dragons module

This is a module which allows wrapper access to the functions within
the SpaceX API to return information about the dragon craft

This file is imported as a module and contains the following
functions:

    * dragon - returns all dragon information
    * one - returns information about a specific dragon craft

"""
from . import urldata
from . import utils


def dragons(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about all dragon craft

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the dragon craft
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_dragons
    return utils.makeRequest(requestUrl, timeOut, parameters)


def one(dragon='', parameters='', timeOut=1):
    """

    :type dragon: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about a specific dragon craft

    Parameters
    ----------
    dragon : name of the dragon craft of interest
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of the dragon craft
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_dragons + \
        "/" + dragon
    return utils.makeRequest(requestUrl, timeOut, parameters)



import sys

sys.path.append('../')

from bs4 import BeautifulSoup
import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *

api_one_ship_data = ''
api_all_ships_data = ''


def getShipPosition(ship_url):
    soup = BeautifulSoup(makeHTTP(ship_url), 'html.parser')
    for link in soup.find_all('a'):
        lnk = link.get('href')
        if str(lnk).startswith('/en/ais/home/centerx'):
            extracted_text = lnk.replace('center', '')\
                .replace('/en/ais/home/', '')\
                .replace('/y', '')
            extracted_text = \
                extracted_text[2:extracted_text.find('/zoom')]\
                .split(':')
    position = '{"latitude": ' + extracted_text[1] \
               + ', "longitude": ' \
               + extracted_text[0] + '}'
    return position

@pytest.mark.skip("Skip one ship position - variable data (time sensitive)")
def test_one_ship_position():
    one_data = ''
    one_result = alphaOrder(
        readJSONFile('ships/oneshipAMERICANCHAMPION.json'))
    try:
        one_data = alphaOrder(
            spacexpython.ships.one('AMERICANCHAMPION', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on ships.one('AMERICANCHAMPION")
    one_result = getShipPosition(json.loads(one_data)['url'])
    file_lat = json.loads(one_data)['position']['latitude']
    file_long = json.loads(one_data)['position']['longitude']
    api_lat = json.loads(one_result)['latitude']
    api_long = json.loads(one_result)['longitude']
    assert (((minus_percent(1, file_lat) <= api_lat <= plus_percent(1, file_lat)) or (minus_percent(1, file_long) <= api_long <= plus_percent(1, file_long)))   # noqa: E501
        or ((minus_percent(1, file_lat) >= api_lat >= plus_percent(1, file_lat)) or (minus_percent(1, file_long) >= api_long >= plus_percent(1, file_long)))    # noqa: E501
        or ((minus_percent(1, file_lat) >= api_lat >= plus_percent(1, file_lat)) or (minus_percent(1, file_long) <= api_long <= plus_percent(1, file_long)))    # noqa: E501
        or ((minus_percent(1, file_lat) <= api_lat <= plus_percent(1, file_lat)) or (minus_percent(1, file_long) >= api_long >= plus_percent(1, file_long))))   # noqa: E501


def test_all_ships():
    api_all_ships_data = ''
    ships_result = keyOrder(alphaOrder(
        readJSONFile('ships/all.json')), 'ship_id')
    try:
        api_all_ships_data = alphaOrder(
            spacexpython.ships.ships())
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on setup of ships test suite")
    api_all_ships_data = keyOrder(api_all_ships_data, 'ship_id')
    assert api_all_ships_data == ships_result



import sys
sys.path.append('../')

import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


@pytest.mark.skip(reason="Skip - history changes over time")
def test_history():
    history_data = ''
    history_result = keyOrder(alphaOrder(
        readJSONFile('history/all.json')), 'id')
    try:
        history_data = alphaOrder(
            spacexpython.history.history('', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on history.allhistory")
    assert history_data == history_result


def test_one_history():
    one_data = ''
    one_result = alphaOrder(
        readJSONFile('history/one.json'))
    try:
        one_data = alphaOrder(spacexpython.history.one('1', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on history.one('dragon1")
    assert one_data == one_result
